package com.sun.jmx.remote.internal;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.rmi.NoSuchObjectException;
import java.rmi.Remote;
import java.util.Properties;
import java.io.IOException;

public final class IIOPHelper
{
    private static final String IMPL_CLASS = "com.sun.jmx.remote.protocol.iiop.IIOPProxyImpl";
    private static final IIOPProxy proxy;
    
    private IIOPHelper() {
    }
    
    public static boolean isAvailable() {
        return IIOPHelper.proxy != null;
    }
    
    private static void ensureAvailable() {
        if (IIOPHelper.proxy == null) {
            throw new AssertionError("Should not here");
        }
        return;
    }
    
    public static boolean isStub(Object o) {
        return IIOPHelper.proxy != null && IIOPHelper.proxy.isStub(o);
    }
    
    public static Object getDelegate(Object o) {
        ensureAvailable();
        return IIOPHelper.proxy.getDelegate(o);
    }
    
    public static void setDelegate(Object o, Object o2) {
        ensureAvailable();
        IIOPHelper.proxy.setDelegate(o, o2);
    }
    
    public static Object getOrb(Object o) {
        ensureAvailable();
        return IIOPHelper.proxy.getOrb(o);
    }
    
    public static void connect(Object o, Object o2) throws IOException {
        if (IIOPHelper.proxy == null) {
            throw new IOException("Connection to ORB failed, RMI/IIOP not available");
        }
        IIOPHelper.proxy.connect(o, o2);
        return;
    }
    
    public static boolean isOrb(Object o) {
        return IIOPHelper.proxy != null && IIOPHelper.proxy.isOrb(o);
    }
    
    public static Object createOrb(String[] array, Properties properties) throws IOException {
        if (IIOPHelper.proxy == null) {
            throw new IOException("ORB initialization failed, RMI/IIOP not available");
        }
        return IIOPHelper.proxy.createOrb(array, properties);
    }
    
    public static Object stringToObject(Object o, String s) {
        ensureAvailable();
        return IIOPHelper.proxy.stringToObject(o, s);
    }
    
    public static String objectToString(Object o, Object o2) {
        ensureAvailable();
        return IIOPHelper.proxy.objectToString(o, o2);
    }
    
    public static <T> T narrow(Object o, Class<T> clazz) {
        ensureAvailable();
        return IIOPHelper.proxy.narrow(o, clazz);
    }
    
    public static void exportObject(Remote remote) throws IOException {
        if (IIOPHelper.proxy == null) {
            throw new IOException("RMI object cannot be exported, RMI/IIOP not available");
        }
        IIOPHelper.proxy.exportObject(remote);
        return;
    }
    
    public static void unexportObject(Remote remote) throws IOException {
        if (IIOPHelper.proxy == null) {
            throw new NoSuchObjectException("Object not exported");
        }
        IIOPHelper.proxy.unexportObject(remote);
        return;
    }
    
    public static Remote toStub(Remote remote) throws IOException {
        if (IIOPHelper.proxy == null) {
            throw new NoSuchObjectException("Object not exported");
        }
        return IIOPHelper.proxy.toStub(remote);
    }
    
    static {
        proxy = (IIOPProxy)AccessController.doPrivileged(new PrivilegedAction<IIOPProxy>() {
            @Override
            public IIOPProxy run() {
                try {
                    return (IIOPProxy)Class.forName("com.sun.jmx.remote.protocol.iiop.IIOPProxyImpl", true, IIOPHelper.class.getClassLoader()).newInstance();
                }
                catch (ClassNotFoundException ex) {
                    return null;
                }
                catch (InstantiationException detailMessage) {
                    throw new AssertionError(detailMessage);
                }
                catch (IllegalAccessException detailMessage2) {
                    throw new AssertionError(detailMessage2);
                }
            }
        });
    }
}
